package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IWorkCenterContainerLocationService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterContainerLocation;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

/**
 * @Author: wangjie
 * @CreateDate: 2021/01/19 15:37
 * @Description:
 **/
@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "产线托盘位置控制层")
public class WorkCenterContainerLocationController extends MesBaseController {

    @Autowired
    private IWorkCenterContainerLocationService workCenterContainerLocationService;

    @GetMapping(value = "/work-center-container-location/query")
    @ApiOperation(value = "按条件分页查询产线托盘位置信息")
    public ResultBean queryWorkCenterContainerLocationByPager(MesWorkCenterContainerLocation workCenterContainerLocation, Pager pager) {
        try {
            workCenterContainerLocation.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(workCenterContainerLocationService.queryWorkCenterContainerLocationByPager(workCenterContainerLocation, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/work-center-container-location/insert")
    @ApiOperation(value = "新增产线托盘位置信息")
    public ResultBean insertWorkCenterContainerLocation(MesWorkCenterContainerLocation workCenterContainerLocation) {
        try {
            ValidatorBean.beginValid(workCenterContainerLocation)
                    .notNull(MesExtConstWords.WORK_CENTER_CODE, workCenterContainerLocation.getWorkCenterCode())
                    .notNull(MesExtConstWords.CT_LOCATION_NO, workCenterContainerLocation.getCtLocationNo())
                    .notNull(MesExtConstWords.LOCATION_TYPE, workCenterContainerLocation.getLocationType());

            workCenterContainerLocation.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            workCenterContainerLocation.setCreateUser(AuthUtil.getSessionUser().getUserName());
            workCenterContainerLocationService.insertWorkCenterContainerLocation(workCenterContainerLocation);

            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/work-center-container-location/update")
    @ApiOperation(value = "更新产线托盘位置信息")
    public ResultBean updateWorkCenterContainerLocation(MesWorkCenterContainerLocation workCenterContainerLocation) {
        try {
            ValidatorBean.beginValid(workCenterContainerLocation)
                    .notNull(MesExtConstWords.ID, workCenterContainerLocation.getId())
                    .notNull(MesExtConstWords.CT_LOCATION_NO, workCenterContainerLocation.getCtLocationNo());

            workCenterContainerLocation.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            workCenterContainerLocation.setModifyUser(AuthUtil.getSessionUser().getUserName());
            workCenterContainerLocationService.updateWorkCenterContainerLocation(workCenterContainerLocation);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/work-center-container-location/status-update")
    @ApiOperation(value = "根据id启用或禁用产线托盘位置信息")
    public ResultBean updateWorkCenterContainerLocationValidById(Long id, Integer status) {
        try {
            if (StringUtils.isEmpty(id)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }
            workCenterContainerLocationService.updateWorkCenterContainerLocationValidById(id, status, AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName());
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
