package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesFourMChange;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesFourMChangeRepository;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesWorkCenterExtRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCellRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 4M变化导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class SxFourMChangeExcelService implements IExcelImportExtService {

    @Autowired
    private MesFourMChangeRepository fourMChangeRepository;

    @Autowired
    private MesWorkCenterExtRepository workCenterExtRepository;

    @Autowired
    private MesWorkCellRepository workCellRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 4M变化导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractMesFourMChange(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelMesFourMChange(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 4M变化导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractMesFourMChange(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesFourMChange集合
        List<MesFourMChange> fourMChangeList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的生产线代码+工位代码+管理类型唯一校验，key：workCenterCode&workCellCode&manageType，value：row
        Map<String, String> existMap = new HashMap<>();

        //查询4M变化表
        List<MesFourMChange> fourMChanges = fourMChangeRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesFourMChange>> fourMChangeMap = CollectionUtils.isEmpty(fourMChanges) ? null :
                fourMChanges.stream().filter(o -> null != o)
                        .collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode() + "&" + o.getManageType()));

        // 查询产线表
        Map<String, Object> workCenterCodeEntityMap;
        try {
            workCenterCodeEntityMap = MesCommonUtil.getCodeEntityMap(workCenterExtRepository, MesExtConstWords.WORK_CENTER_CODE, organizeCode, "产线表");
        } catch (Exception e) {
            workCenterCodeEntityMap = null;
        }

        // 查询工位表
        List<MesWorkCell> workCellList = workCellRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesWorkCell>> workCellCodeEntityMap = CollectionUtils.isEmpty(workCellList) ? null :
                workCellList.stream().filter(o -> null != o).collect(Collectors.groupingBy(o -> o.getWorkCenterCode() + "&" + o.getWorkCellCode()));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //生产线代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String workCenterCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //工位代码
                String workCellCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    workCellCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //管理类型
                String manageType = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    manageType = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //变化前内容
                String beforeContent = null;
                try {
                    row.getCell(3).setCellType(CellType.STRING);
                    beforeContent = row.getCell(3, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //变化后内容
                String afterContent = null;
                try {
                    row.getCell(4).setCellType(CellType.STRING);
                    afterContent = row.getCell(4, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                //校验生产线代码
                if (CollectionUtils.isEmpty(workCenterCodeEntityMap) || !workCenterCodeEntityMap.containsKey(workCenterCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据无效;";
                }

                //校验工位代码
                if (StringUtils.isEmpty(workCellCode)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (StringUtils.isEmpty(errorInfo)) {
                        if (CollectionUtils.isEmpty(workCellCodeEntityMap) || !workCellCodeEntityMap.containsKey(workCenterCode + "&" + workCellCode)) {
                            errorNum++;
                            cellNum += "A+B;";
                            errorInfo += "第A+B列数据无效;";
                        }
                    }
                }

                //校验管理类型
                if (StringUtils.isEmpty(manageType)) {
                    errorNum ++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                } else {
                    if (StringUtils.isEmpty(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType))) {
                        errorNum ++;
                        cellNum += "C;";
                        errorInfo += "第C列数据无效;";
                    }
                }

                boolean isExist = false;
                //校验生产线代码+工位代码+管理类型不重复
                String key = workCenterCode + "&" + workCellCode + "&" + MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType);
                if (!existMap.containsKey(key)) {
                    existMap.put(key, String.valueOf(rowNum));
                    //校验 生产线代码+工位代码+管理类型 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(fourMChangeMap) && fourMChangeMap.containsKey(key)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A+B+C;";
                    errorInfo += "第A+B+C列数据跟第" + existMap.get(key) + "行的第A+B+C列数据重复;";
                }

                //校验变化前内容
                if (!StringUtils.isEmpty(beforeContent) && beforeContent.length() > 255) {
                    errorNum ++;
                    cellNum += "D;";
                    errorInfo += "第D列数据长度不能超过255个字符;";
                }

                //校验变化后内容
                if (!StringUtils.isEmpty(afterContent) && afterContent.length() > 255) {
                    errorNum ++;
                    cellNum += "E;";
                    errorInfo += "第E列数据长度不能超过255个字符;";
                }

                //没有错误的时候，封装MesFourMChange
                if (errorNum == 0) {
                    MesFourMChange fourMChange;
                    if(isExist){
                        fourMChange = fourMChangeMap.get(key).get(0);
                        ConvertBean.serviceModelUpdate(fourMChange, userName);
                    }else{
                        fourMChange = new MesFourMChange();
                        ConvertBean.serviceModelInitialize(fourMChange, userName);
                    }
                    fourMChange.setWorkCenterCode(workCenterCode);
                    fourMChange.setWorkCellCode(workCellCode);
                    fourMChange.setManageType(MesExtEnumUtil.MANAGE_TYPE.descriptionOfValue(manageType));
                    fourMChange.setBeforeContent(beforeContent);
                    fourMChange.setAfterContent(afterContent);
                    fourMChange.setOrganizeCode(organizeCode);

                    fourMChangeList.add(fourMChange);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : fourMChangeList);
        return excelImportResultModel;
    }

    /**
     * 4M变化扩展导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelMesFourMChange(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesFourMChange> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesFourMChange fourMChange : excelList) {
                    if(StringUtils.isEmpty(fourMChange.getId())){
                        fourMChangeRepository.insert(fourMChange);
                    }else{
                        fourMChangeRepository.save(fourMChange);
                    }
                }
            }
        }
    }
}
